#!/usr/bin/env python3
from django.core.management.base import BaseCommand
from arkindex_common.enums import DataImportMode
from arkindex.project.argparse import DataImportArgument
from arkindex.dataimport.git import GitFlow
import tempfile
import os
import logging

logging.basicConfig(
    level=logging.INFO,
    format='[%(levelname)s] %(message)s',
)
logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = 'Import files from a data import'

    def add_arguments(self, parser):
        parser.add_argument(
            'data_import',
            type=DataImportArgument(),
            help='ID of the DataImport to run'
        )

    def handle(self, *args, data_import=None, **options):
        assert data_import.mode == DataImportMode.Repository, \
            'Only Repository imports are supported via this command'

        # Use shared directory when running in docker
        # Fallback to a temp directory while developing
        task_dir = os.path.join(
            os.environ.get('PONOS_DATA', tempfile.mkdtemp(suffix='-ponos')),
            'current',
        )
        assert os.path.isdir(task_dir), 'Invalid task dir {}'.format(task_dir)
        logger.info('Using task dir: {}'.format(task_dir))

        git = GitFlow(data_import, task_dir)
        git.run()
