from django.contrib import admin
from django import forms
from django_admin_hstore_widget.forms import HStoreFormField
from arkindex.documents.models import \
    Corpus, Element, Transcription, MetaData, MLClass, Classification, \
    DataSource, Entity, EntityRole, EntityLink, Region, RegionElement
from arkindex.dataimport.models import Event
from enumfields.admin import EnumFieldListFilter


class CorpusAdmin(admin.ModelAdmin):
    list_display = ('id', 'name', 'public', )


class DataSourceAdmin(admin.ModelAdmin):
    list_display = ('id', 'type', 'slug', 'revision', 'internal')
    list_filter = [('type', EnumFieldListFilter), 'internal']
    readonly_fields = ('id', )


class EventInline(admin.TabularInline):
    model = Event


class ClassificationInline(admin.TabularInline):
    model = Classification


class MetaDataAdmin(admin.ModelAdmin):
    list_display = ('id', 'type', 'revision')
    readonly_fields = ('id', 'revision')
    raw_id_fields = ('element', 'entity', )


class MetaDataInline(admin.TabularInline):
    model = MetaData
    raw_id_fields = ('entity', )


class ElementAdmin(admin.ModelAdmin):
    list_display = ('id', 'name', 'type', 'corpus', )
    list_filter = [('type', EnumFieldListFilter), 'corpus']
    fields = ('id', 'type', 'name', 'corpus')
    readonly_fields = ('id', )
    search_fields = ('name', )
    inlines = (MetaDataInline, EventInline, ClassificationInline)


class TranscriptionAdmin(admin.ModelAdmin):
    list_display = ('id', 'text', 'score', 'element', )
    list_filter = [('type', EnumFieldListFilter), 'source']
    fields = ('id', 'text', 'score', 'element', 'zone', 'source', )
    readonly_fields = ('id', )
    raw_id_fields = ('element', 'zone', )


class MLClassAdmin(admin.ModelAdmin):
    list_display = ('id', 'name', 'corpus')
    list_filter = ('corpus',)
    fields = ('name', 'corpus')


class EntityMetaForm(forms.ModelForm):
    metas = HStoreFormField()


class EntityLinkInLine(admin.TabularInline):
    model = EntityLink
    fk_name = 'parent'
    raw_id_fields = ('child', )


class EntityAdmin(admin.ModelAdmin):
    list_display = ('id', 'name', 'type')
    list_filter = [('type', EnumFieldListFilter), 'corpus']
    readonly_fields = ('id', )
    search_fields = ('name', )
    inlines = (EntityLinkInLine, )
    form = EntityMetaForm


class EntityRoleAdmin(admin.ModelAdmin):
    list_display = ('id', 'parent_name', 'child_name')
    list_filter = ['corpus']
    readonly_fields = ('id', )


class RegionElementInline(admin.TabularInline):
    model = RegionElement
    fields = ('region', 'element', 'ordering')
    raw_id_fields = ('region', 'element')


class RegionAdmin(admin.ModelAdmin):
    list_display = ('id', 'ml_class', 'source')
    readonly_fields = ('id', )
    raw_id_fields = ('zone', )
    inlines = (RegionElementInline, )


admin.site.register(Corpus, CorpusAdmin)
admin.site.register(DataSource, DataSourceAdmin)
admin.site.register(Element, ElementAdmin)
admin.site.register(Transcription, TranscriptionAdmin)
admin.site.register(MLClass, MLClassAdmin)
admin.site.register(MetaData, MetaDataAdmin)
admin.site.register(Entity, EntityAdmin)
admin.site.register(EntityRole, EntityRoleAdmin)
admin.site.register(Region, RegionAdmin)
