from django.db.models import Q, Count
from django_filters import rest_framework as filters

from arkindex.documents.models import MLClass, Element, ElementType, Page, ClassificationState


class MLClassVolumeChoiceFilter(filters.ModelChoiceFilter):
    def get_queryset(self, request):
        return Element.objects.filter(type=ElementType.Volume, corpus=request.corpus)


class CorpusMLClassFilter(filters.FilterSet):
    volume = MLClassVolumeChoiceFilter(method='filter_volume', label='Volume')

    def filter_volume(self, queryset, name, value):
        return queryset.filter(classifications__element__in=Page.objects.get_descending(value.pk)).distinct()

    class Meta:
        model = MLClass
        fields = ['volume']


class ElementsMLClassFilter(filters.FilterSet):
    ml_class = filters.ModelChoiceFilter(
        queryset=MLClass.objects.all(), field_name='classifications__ml_class', label='Class')
    ml_class_unvalidated = filters.BooleanFilter(
        field_name='classifications__ml_class', method='filter_ml_class_unvalidated')

    def filter_ml_class_unvalidated(self, queryset, name, value):
        base_qs = queryset.annotate(
            num_validated=Count('classifications', filter=Q(classifications__state=ClassificationState.Validated)))
        if value:
            return base_qs.filter(num_validated=0)
        return base_qs.exclude(num_validated=0)

    class Meta:
        model = Element
        fields = ['ml_class', 'ml_class_unvalidated']
