from django.apps import AppConfig
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
import os
import pkg_resources
import arkindex


class DocumentsConfig(AppConfig):
    name = 'arkindex.documents'

    def _package_version(self, name):
        try:
            with open(os.path.join(settings.BASE_DIR, '..', 'VERSION')) as f:
                return f.read()
        except (OSError, AttributeError, ImproperlyConfigured):
            # File not found or settings module not ready
            pass

        try:
            return pkg_resources.get_distribution(name).version
        except pkg_resources.ResolutionError:
            pass

        raise RuntimeError("Could not fetch '{}' version".format(name))

    def ready(self):
        from arkindex.project import checks  # noqa: F401
        from arkindex.documents import signals  # noqa: F401
        arkindex.VERSION = self._package_version('arkindex')
