from pathlib import Path
from django.conf import settings
from ponos.management.base import PonosCommand, FolderArgument
from arkindex.project.argparse import CorpusArgument
from arkindex.documents.acts import ActsImporter
import logging

logging.basicConfig(
    format='%(asctime)s [%(levelname)s] %(message)s',
    level=logging.INFO,
)


class Command(PonosCommand):
    help = "Import surfaces and acts from CSV files"
    docker_image = settings.ARKINDEX_APP_IMAGE
    base_recipe = settings.PONOS_RECIPE

    def add_arguments(self, parser):
        super().add_arguments(parser)
        parser.add_argument(
            'csv',
            help='Path to a folder containing CSV files with surface-act associations, named after the volumes.',
            type=FolderArgument(),
        )
        parser.add_argument(
            '--corpus',
            type=CorpusArgument(),
            help='ID or part of the name of a corpus to import surfaces into.',
            required=True,
        )

    def run(self, csv, corpus, **options):
        csv = Path(csv)
        for path in csv.rglob('*.csv'):
            try:
                importer = ActsImporter(path, corpus)
                importer.run()
            except Exception as e:
                self.stdout.write(self.style.ERROR(str(e)))
