from django.urls import path
from django.views.generic.base import RedirectView

from arkindex.documents.api.elements import (
    ElementsList, RelatedElementsList, ElementRetrieve, ElementPages,
    CorpusList, CorpusRetrieve, ActEdit, PageDetails, SurfaceDetails,
    ElementTranscriptions, ElementsCreate, ElementRegions, RegionDetails, RegionCreate,
    ElementNeighbors,
)
from arkindex.documents.api.search import ElementSearch, EntitySearch
from arkindex.documents.api.ml import (
    ClassificationCreate, ClassificationValidate, ClassificationReject, ClassificationBulk,
    TranscriptionCreate, TranscriptionBulk, PageXmlTranscriptionsImport,
)
from arkindex.documents.api.entities import (
    CorpusRoles, EntityDetails, EntityCreate, EntityLinkCreate, EntityElements, MLClassList,
    CorpusMLClassList, TranscriptionEntityCreate, TranscriptionEntities, ElementEntities,
    ElementLinks
)
from arkindex.documents.api.iiif import (
    VolumeManifest, ActManifest, PageAnnotationList, PageActAnnotationList, SurfaceAnnotationList,
    TranscriptionSearchAnnotationList,
)
from arkindex.dataimport.api import (
    DataImportsList, DataImportDetails, DataImportRetry, DataImportDemo,
    DataFileList, DataFileRetrieve, DataFileUpload, DataImportFromFiles,
    RepositoryList, RepositoryRetrieve, RepositoryStartImport, DataFileCreate,
    GitRepositoryImportHook, AvailableRepositoriesList, ElementHistory, MLToolList,
    ElementWorkflow
)
from arkindex.images.api import ImageCreate, ImageRetrieve
from arkindex.users.api import (
    ProvidersList, CredentialsList, CredentialsRetrieve, OAuthSignIn, OAuthRetry, OAuthCallback,
    UserRetrieve, UserCreate, UserEmailLogin, UserEmailVerification, PasswordReset, PasswordResetConfirm,
)

api = [

    # Elements
    path('elements/<uuid:pk>/neighbors/', ElementNeighbors.as_view(), name='elements-neighbors'),
    path('elements/<uuid:pk>/pages/', ElementPages.as_view(), name='volume-pages'),
    path('elements/<uuid:pk>/', RelatedElementsList.as_view(), name='related-elements'),
    path('elements/', ElementsList.as_view(), name='elements'),
    path('elements/create/', ElementsCreate.as_view(), name='elements-create'),
    path('element/<uuid:pk>/', ElementRetrieve.as_view(), name='element-retrieve'),
    path('element/<uuid:pk>/history/', ElementHistory.as_view(), name='element-history'),
    path('element/<uuid:pk>/entities/', ElementEntities.as_view(), name='element-entities'),
    path('element/<uuid:pk>/links/', ElementLinks.as_view(), name='element-links'),
    path('element/<uuid:pk>/regions/', ElementRegions.as_view(), name='element-regions'),
    path('element/<uuid:pk>/region/', RegionCreate.as_view(), name='region-create'),
    path('element/<uuid:pk>/transcriptions/', ElementTranscriptions.as_view(), name='element-transcriptions'),
    path('element/<uuid:pk>/workflow/', ElementWorkflow.as_view(), name='element-workflow'),
    path('page/<uuid:pk>/', PageDetails.as_view(), name='page-details'),
    path(
        'page/<uuid:pk>/transcriptions/xml/',
        PageXmlTranscriptionsImport.as_view(),
        name='pagexml-transcriptions',
    ),
    path('surface/<uuid:pk>/', SurfaceDetails.as_view(), name='surface-details'),
    path('region/<uuid:pk>/', RegionDetails.as_view(), name='region-details'),
    path('corpus/', CorpusList.as_view(), name='corpus'),
    path('corpus/<uuid:pk>/', CorpusRetrieve.as_view(), name='corpus-retrieve'),
    path('corpus/<uuid:pk>/classes/', CorpusMLClassList.as_view(), name='corpus-classes'),
    path('corpus/<uuid:pk>/roles/', CorpusRoles.as_view(), name='corpus-roles'),

    # Moderation
    path('ml-classes/', MLClassList.as_view(), name='mlclass-list'),
    path('classifications/', ClassificationCreate.as_view(), name='classification-create'),
    path('classifications/<uuid:pk>/validate/', ClassificationValidate.as_view(), name='classification-validate'),
    path('classifications/<uuid:pk>/reject/', ClassificationReject.as_view(), name='classification-reject'),

    # Manifests
    path('manifest/<uuid:pk>/pages/', VolumeManifest.as_view(), name='volume-manifest'),
    path('manifest/<uuid:pk>/act/', ActManifest.as_view(), name='act-manifest'),
    path('manifest/<uuid:pk>/transcriptions/', PageAnnotationList.as_view(), name='page-transcription-manifest'),
    path('manifest/<uuid:pk>/acts/', PageActAnnotationList.as_view(), name='page-act-manifest'),
    path('manifest/<uuid:pk>/surface/', SurfaceAnnotationList.as_view(), name='surface-manifest'),
    path('manifest/<uuid:pk>/search/', TranscriptionSearchAnnotationList.as_view(), name='ts-search-manifest'),

    # Placeholder URLs for IIIF IDs
    path(
        'manifest/<uuid:pk>/sequence/',
        RedirectView.as_view(pattern_name='api:volume-manifest', permanent=False),
        name='sequence-manifest',
    ),
    path(
        'manifest/<uuid:pk>/canvas/',
        RedirectView.as_view(pattern_name='api:page-act-manifest', permanent=False),
        name='canvas-manifest',
    ),
    path(
        'manifest/<uuid:page_pk>/transcriptions/<uuid:transcription_pk>/',
        RedirectView.as_view(pattern_name='api:page-transcription-manifest', permanent=False),
        name='transcription-manifest',
    ),
    path(
        'manifest/<uuid:page_pk>/surfaces/<uuid:surface_pk>/',
        RedirectView.as_view(pattern_name='api:page-act-manifest', permanent=False),
        name='surface-manifest',
    ),

    # Search engines
    path('elements/search/', ElementSearch.as_view(), name='element-search'),
    path('entity/search/', EntitySearch.as_view(), name='entity-search'),

    # Edit acts
    path('act/<uuid:pk>/', ActEdit.as_view(), name='act-edit'),

    # Ingest transcriptions
    path('transcription/', TranscriptionCreate.as_view(), name='transcription-create'),
    path('transcription/bulk/', TranscriptionBulk.as_view(), name='transcription-bulk'),
    path('classification/bulk/', ClassificationBulk.as_view(), name='classification-bulk'),

    # Ingest and retrieve entities
    path('entity/', EntityCreate.as_view(), name='entity-create'),
    path('entity/link/', EntityLinkCreate.as_view(), name='entity-link-create'),
    path('entity/<uuid:pk>/', EntityDetails.as_view(), name='entity-details'),
    path('entity/<uuid:pk>/elements/', EntityElements.as_view(), name='entity-elements'),
    path('transcription/<uuid:pk>/entity/', TranscriptionEntityCreate.as_view(), name='transcription-entity-create'),
    path('transcription/<uuid:pk>/entities/', TranscriptionEntities.as_view(), name='transcription-entities'),

    # Git import workflows
    path('imports/repos/', RepositoryList.as_view(), name='repository-list'),
    path('imports/repos/<uuid:pk>/', RepositoryRetrieve.as_view(), name='repository-retrieve'),
    path('imports/repos/<uuid:pk>/start/', RepositoryStartImport.as_view(), name='repository-import'),
    path('imports/repos/search/<uuid:pk>/', AvailableRepositoriesList.as_view(), name='available-repositories'),
    path('imports/hook/<uuid:pk>/', GitRepositoryImportHook.as_view(), name='import-hook'),

    # Import workflows
    path('imports/', DataImportsList.as_view(), name='import-list'),
    path('imports/fromfiles/', DataImportFromFiles.as_view(), name='import-from-files'),
    path('imports/mltools/', MLToolList.as_view(), name='ml-tool-list'),
    path('imports/<uuid:pk>/', DataImportDetails.as_view(), name='import-details'),
    path('imports/<uuid:pk>/retry/', DataImportRetry.as_view(), name='import-retry'),
    path('imports/demo/<uuid:pk>/', DataImportDemo.as_view(), name='import-demo'),
    path('imports/files/<uuid:pk>/', DataFileList.as_view(), name='file-list'),
    path('imports/files/create/', DataFileCreate.as_view(), name='file-create'),
    path('imports/file/<uuid:pk>/', DataFileRetrieve.as_view(), name='file-retrieve'),
    path('imports/upload/<uuid:pk>/', DataFileUpload.as_view(), name='file-upload'),

    # Image management
    path('image/', ImageCreate.as_view(), name='image-create'),
    path('image/<uuid:pk>/', ImageRetrieve.as_view(), name='image-retrieve'),

    # Manage OAuth integrations
    path('oauth/providers/', ProvidersList.as_view(), name='providers-list'),
    path('oauth/providers/<provider>/signin/', OAuthSignIn.as_view(), name='oauth-signin'),
    path('oauth/providers/<provider>/callback/', OAuthCallback.as_view(), name='oauth-callback'),
    path('oauth/credentials/', CredentialsList.as_view(), name='credentials-list'),
    path('oauth/credentials/<uuid:pk>/', CredentialsRetrieve.as_view(), name='credentials-retrieve'),
    path('oauth/credentials/<uuid:pk>/retry/', OAuthRetry.as_view(), name='oauth-retry'),

    # Authentication
    path('user/', UserRetrieve.as_view(), name='user-retrieve'),
    path('user/new/', UserCreate.as_view(), name='user-new'),
    path('user/login/', UserEmailLogin.as_view(), name='user-login'),
    path('user/token/', UserEmailVerification.as_view(), name='user-token'),
    path('user/password-reset/', PasswordReset.as_view(), name='password-reset'),
    path('user/password-reset/confirm/', PasswordResetConfirm.as_view(), name='password-reset-confirm'),
]
