from arkindex.documents.models import Element, ElementPath, ElementType
from arkindex.project.tests import FixtureTestCase


class TestArrayField(FixtureTestCase):
    """
    Test the enhanced ArrayField
    """

    def setUp(self):
        """
        D -> C -> B -> A
        """
        self.a, self.b, self.c, self.d = (
            Element.objects.create(corpus=self.corpus, name=str(i), type=ElementType.Volume)
            for i in range(4)
        )
        self.d.add_parent(self.c)
        self.c.add_parent(self.b)
        self.b.add_parent(self.a)
        self.assertEqual(ElementPath.objects.filter(element=self.a).count(), 0)
        self.assertEqual(ElementPath.objects.get(element=self.b).path, [self.a.id])
        self.assertEqual(ElementPath.objects.get(element=self.c).path, [self.a.id, self.b.id])
        self.assertEqual(ElementPath.objects.get(element=self.d).path, [self.a.id, self.b.id, self.c.id])

    def test_last(self):
        """
        Test the __last transform
        """
        self.assertEqual(ElementPath.objects.get(path__last__exact=self.a.id).element, self.b)
        self.assertEqual(ElementPath.objects.get(path__last=self.b.id).element, self.c)
        self.assertEqual(ElementPath.objects.filter(path__last__in=[self.b.id, self.c.id]).count(), 2)
