from unittest import TestCase
from arkindex.project.polygon import Point, Polygon
from arkindex.project.serializer_fields import PointField, PolygonField
from rest_framework.serializers import ValidationError


class TestPolygonSerializerField(TestCase):

    def test_pointfield_to_json(self):
        self.assertListEqual(
            PointField().to_representation(Point(13, 37)),
            [13, 37],
        )

    def test_json_to_pointfield(self):
        self.assertEqual(
            PointField().to_internal_value([13, 37]),
            Point(13, 37),
        )

    def test_bad_json_to_pointfield(self):
        with self.assertRaises(ValidationError):
            PointField().to_internal_value([])
        with self.assertRaises(ValidationError):
            PointField().to_internal_value([1, 2, 3])
        with self.assertRaises(ValidationError):
            PointField().to_internal_value(['a', 5])

    def test_polygonfield_to_json(self):
        self.assertListEqual(
            PolygonField().to_representation(Polygon([
                Point(10, 20),
                Point(30, 40),
                Point(-40, 50),
            ])),
            [
                [10, 20],
                [30, 40],
                [-40, 50],
                [10, 20],
            ]
        )

    def test_json_to_polygonfield(self):
        self.assertEqual(
            PolygonField().to_internal_value([
                [10, 20],
                [30, 40],
                [-40, 50],
            ]), Polygon([
                Point(10, 20),
                Point(30, 40),
                Point(-40, 50),
                Point(10, 20),
            ])
        )

    def test_bad_json_to_polygonfield(self):
        with self.assertRaises(ValidationError):
            PolygonField().to_internal_value([])
        with self.assertRaises(ValidationError):
            PolygonField().to_internal_value([1, 2, 3])
        with self.assertRaises(ValidationError):
            PolygonField().to_internal_value([[1, 2]])
